package xyz.dcdevelop.uni.y2s1.ads;

import java.util.*;
import java.util.stream.Stream;
import java.util.stream.StreamSupport;

public class LinkedList<T> implements Iterable<T> {
    public Stream<T> stream() {
        return StreamSupport.stream(Spliterators.spliteratorUnknownSize(iterator(), Spliterator.DISTINCT), false);
    }

    @Override
    public Iterator<T> iterator() {
        return new LLIterator(root);
    }

    protected class Node {
        private T value;
        private Node next;

        protected Node(T value) {
            this.value = value;
            next = null;
        }
    }

    public class LLIterator implements Iterator<T> {
        private Node currentNode;
        private Node initialNode;

        public LLIterator(Node initialNode) {
            this.initialNode = initialNode;
            currentNode = null;
        }

        @Override
        public boolean hasNext() {
            return initialNode != null || currentNode != null && currentNode.next != null;
        }

        @Override
        public T next() {
            if (!hasNext()) {
                throw new NoSuchElementException();
            }
            if (initialNode != null) {
                currentNode = initialNode;
                initialNode = null;
            }
            else {
                currentNode = currentNode.next;
            }
            return currentNode.value;
        }
    }

    private Node root;
    private Node last;

    public void add(T element) {
        Node n = new Node(element);
        if (last == null) {
            root = n;
        }
        else {
            last.next = n;
        }
        last = n;
    }

    public void reverseList() {
        reverseList(null, root);
    }

    private void reverseList(Node previousNode, Node currentNode) {
        if (currentNode.next == null) {
            root = currentNode;
        }
        else {
            reverseList(currentNode, currentNode.next);
        }
        currentNode.next = previousNode;
    }
}
